package simulation_Lab12Version;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

/**
 * This class encapsulates functionality for the control of an animation.
 * 
 * The controller advances the animation from one frame to the next. This class
 * encapsulates functionality as an ActionListener, so that objects of this type
 * can listen for events that trigger the drawing of the successive frames.
 * 
 * @author mb
 * 
 */
public class AnimationController_Lab12 implements ActionListener, KeyListener {

	// attributes that concern the drawing environment
	private View theView;

	// attribute for what is drawn on-screen (encapsulates the sprite(s))
	private Model animationModel;

	private boolean isMoveRight = false;
	private boolean isMoveLeft = false;
	private boolean isMoveUp = false;
	private boolean isMoveDown = false;
	private boolean isShooting = false;
	private boolean isRotateRight = false;
	private boolean isRotateLeft = false;

	/**
	 * Instantiates a controller that is installed on the passed View and will,
	 * in response to user events, mutate the passed Model.
	 * 
	 * @param model
	 */
	public AnimationController_Lab12(Model model, View view) {

		// initialize the attribute(s) that concern that which is drawn on each
		// frame
		animationModel = model;

		// assign the view
		theView = view;

		// one reason we pass the gameCanvas to the controller is so that
		// we can install a listener on it
		theView.addKeyListener(this);
	}

	public void actionPerformed(ActionEvent ae) {
		// System.out.println("advancing next state");
		// delegate to the model to determine the next state of the animation
		animationModel.advanceNextState();
		if (isMoveRight) {
			animationModel.moveShooterRight();
		}
		if (isMoveDown) {
			animationModel.moveShooterDown();
		}
		if (isMoveLeft) {
			animationModel.moveShooterLeft();
		}
		if (isMoveUp) {
			animationModel.moveShooterUp();
		}
		if (isShooting) {
			animationModel.fireShooter();
		}
		if (isRotateRight) {
			animationModel.rotateShooterRight();
		}
		if (isRotateLeft) {
			animationModel.rotateShooterLeft();
		}
	}

	public void keyTyped(KeyEvent e) {
	}

	public void keyPressed(KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_RIGHT) {
			isMoveRight = true;
		}
		if (e.getKeyCode() == KeyEvent.VK_DOWN) {
			isMoveDown = true;
		}
		if (e.getKeyCode() == KeyEvent.VK_LEFT) {
			isMoveLeft = true;
		}
		if (e.getKeyCode() == KeyEvent.VK_UP) {
			isMoveUp = true;
		}
		if (e.getKeyCode() == KeyEvent.VK_SPACE) {
			isShooting = true;
		}
		if (e.getKeyCode() == KeyEvent.VK_PERIOD) {
			isRotateRight = true;
		}
		if (e.getKeyCode() == KeyEvent.VK_COMMA) {
			isRotateLeft = true;
		}

	}

	public void keyReleased(KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_A) {
			animationModel.addSprite();
		}
		if (e.getKeyCode() == KeyEvent.VK_RIGHT) {
			isMoveRight = false;
		}
		if (e.getKeyCode() == KeyEvent.VK_DOWN) {
			isMoveDown = false;
		}
		if (e.getKeyCode() == KeyEvent.VK_LEFT) {
			isMoveLeft = false;
		}
		if (e.getKeyCode() == KeyEvent.VK_UP) {
			isMoveUp = false;
		}
		if (e.getKeyCode() == KeyEvent.VK_SPACE) {
			isShooting = false;
		}
		if (e.getKeyCode() == KeyEvent.VK_PERIOD) {
			isRotateRight = false;
		}
		if (e.getKeyCode() == KeyEvent.VK_COMMA) {
			isRotateLeft = false;
		}

	}

}
