package sprites;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Toolkit;

import javax.swing.ImageIcon;

import simulation_Lab12Version.UtilityCircle;

/**
 * This class encapsulates a sprite which is an image of a fly. The sprite's
 * initial position is randomly chosen. For each frame, this sprite moves 1 unit
 * down and 1 unit to the right, wrapping around the canvas when the boundaries
 * are encountered.
 * 
 * @author mb
 * 
 */
public class BeetleShooter extends ShooterSprite {

	// overshadow interface's fields
	private int RIGHT_DELTA = 3;
	private int LEFT_DELTA = -3;
	private int DOWN_DELTA = 3;
	private int UP_DELTA = -3;

	// attribute for the on-screen sprite
	private final String IMG_FILENAME = "assets/beetle.png";
	private Image img;
	private final int MAX_IMG_DIM = 100;
	private Dimension imgDim = new Dimension(MAX_IMG_DIM, MAX_IMG_DIM);

	/**
	 * Creates a basic sprite within a game world with the specified dimensions.
	 * 
	 * @param widthOfDrawingRegion
	 *            as described above
	 * @param heightOfDrawingRegion
	 *            as described above
	 */
	public BeetleShooter(int widthOfDrawingRegion, int heightOfDrawingRegion) {
		super(widthOfDrawingRegion, heightOfDrawingRegion);
		img = Toolkit.getDefaultToolkit().getImage(IMG_FILENAME);

		Point p = new Point((int) (-.07 * MAX_IMG_DIM),
				(int) (-0.45 * MAX_IMG_DIM));
		// System.out.println("tip relative to center: " + p);

		setPointOfShooterTipRelativeToSpriteCentre(p);

		Polygon theShape = new Polygon();
		theShape.addPoint(0, 0);
		theShape.addPoint(MAX_IMG_DIM, 0);
		theShape.addPoint(MAX_IMG_DIM, MAX_IMG_DIM);
		theShape.addPoint(0, MAX_IMG_DIM);
		setTheBoundingShape(theShape);

		setAnchorPoint(new Point(widthOfDrawingRegion / 2,
				heightOfDrawingRegion / 2));

	}

	public int getRightDelta() {
		return RIGHT_DELTA;
	}

	public int getLeftDelta() {
		return LEFT_DELTA;
	}

	public int getDownDelta() {
		return DOWN_DELTA;
	}

	public int getUpDelta() {
		return UP_DELTA;
	}

	public void draw(Graphics2D graphics2D) {
		graphics2D.rotate(getAngularDisplacement(), getBounds2D().getCenterX(),
				getBounds2D().getCenterY());
		graphics2D.drawImage(img, getAnchorPoint().x, getAnchorPoint().y,
				(int) imgDim.getWidth(), (int) imgDim.getHeight(), null);
		graphics2D.rotate(-getAngularDisplacement(),
				getBounds2D().getCenterX(), getBounds2D().getCenterY());
	}

	public void moveLeft() {
		Point newPosition = getNewPosition(getAnchorPoint(), getLeftDelta(), 0);
		setAnchorPoint(newPosition);
	}

	public void moveRight() {
		Point newPosition = getNewPosition(getAnchorPoint(), getRightDelta(), 0);
		setAnchorPoint(newPosition);
	}

	public void moveUp() {
		Point newPosition = getNewPosition(getAnchorPoint(), 0, getUpDelta());
		setAnchorPoint(newPosition);
	}

	public void moveDown() {
		Point newPosition = getNewPosition(getAnchorPoint(), 0, getDownDelta());
		setAnchorPoint(newPosition);
	}

	public ProjectileSprite fire() {
		int hypotenuse = 7;

		int yDisplacement = (int) (Math.cos(getAngularDisplacement()) * hypotenuse);
		int xDisplacement = (int) (Math.sin(getAngularDisplacement()) * hypotenuse);

		// System.out.println("fire from tip:" + getShooterTip());
		return new MyProjectileSprite(getWidthOfDrawingRegion(),
				getHeightOfDrawingRegion(), getShooterTip(), xDisplacement,
				-yDisplacement);
	}

}
