package sprites;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Toolkit;

import javax.swing.ImageIcon;

/**
 * This class encapsulates a sprite which is an image of a fly. The sprite's
 * initial position is randomly chosen. For each frame, this sprite moves 1 unit
 * down and 1 unit to the right, wrapping around the canvas when the boundaries
 * are encountered.
 * 
 * @author mb
 * 
 */
public class FlowerSprite extends Sprite {

	// attribute for the on-screen sprite
	private final String IMG_FILENAME = "assets/flower.gif";
	private Image img;
	private final int MAX_IMG_DIM = 50;
	private Dimension imgDim = new Dimension(MAX_IMG_DIM, MAX_IMG_DIM);

	/**
	 * Creates a basic sprite within a game world with the specified dimensions.
	 * 
	 * @param widthOfDrawingRegion
	 *            as described above
	 * @param heightOfDrawingRegion
	 *            as described above
	 */
	public FlowerSprite(int widthOfDrawingRegion, int heightOfDrawingRegion) {
		super(widthOfDrawingRegion, heightOfDrawingRegion);
		img = Toolkit.getDefaultToolkit().getImage(IMG_FILENAME);

		Polygon theShape = new Polygon();
		theShape.addPoint(0, 0);
		theShape.addPoint(MAX_IMG_DIM, 0);
		theShape.addPoint(MAX_IMG_DIM, MAX_IMG_DIM);
		theShape.addPoint(0, MAX_IMG_DIM);
		setTheBoundingShape(theShape);

		setAnchorPoint(new Point(widthOfDrawingRegion / 2,
				heightOfDrawingRegion / 2));

	}

	/**
	 * Draws this sprite on the passed Graphics2D objects. Presently the
	 * explosion is very anti-climatic (it just disappears).
	 */
	public void draw(Graphics2D graphics2D) {
		if (isExploded()) {
			// if this sprite is exploded, don't draw it
		} else {
			graphics2D.drawImage(img, getAnchorPoint().x, getAnchorPoint().y,
					(int) imgDim.getWidth(), (int) imgDim.getHeight(), null);
		}
	}

	public void moveLeft() {
		Point newPosition = getNewPosition(getAnchorPoint(), getLeftDelta(), 0);
		newPosition = addJitter(newPosition);
		setAnchorPoint(newPosition);
	}

	public void moveRight() {
		Point newPosition = getNewPosition(getAnchorPoint(), getRightDelta(), 0);
		newPosition = addJitter(newPosition);
		setAnchorPoint(newPosition);
	}

	public void moveUp() {
		Point newPosition = getNewPosition(getAnchorPoint(), 0, getUpDelta());
		newPosition = addJitter(newPosition);
		setAnchorPoint(newPosition);
	}

	public void moveDown() {
		Point newPosition = getNewPosition(getAnchorPoint(), 0, getDownDelta());
		newPosition = addJitter(newPosition);
		setAnchorPoint(newPosition);
	}

}
