package sprites;

import java.awt.Point;

import simulation_Lab12Version.UtilityCircle;

public abstract class ShooterSprite extends Sprite {
	/*
	 * tip of the shooter, expressed as a displacement from the anchor point of
	 * the bounding box.
	 */
	private Point pointOfShooterTipRelativeToSpriteCentre;

	/*
	 * angular displacement of shooter, expressed in radians around center pivot
	 * point
	 */
	private double angularDisplacement = 0;
	private final double ANGLE_DELTA = Math.PI / 45;

	protected ShooterSprite(int widthOfDrawingRegion, int heightOfDrawingRegion) {
		super(widthOfDrawingRegion, heightOfDrawingRegion);
	}

	public ProjectileSprite fire() {
		return null;
	}

	public double getAngularDisplacement() {
		return angularDisplacement;
	}

	public void setAngularDisplacement(double angularDisplacement) {
		this.angularDisplacement = angularDisplacement;
	}

	protected Point getShooterTip() {
		// derive tip relative to sprite's own origin
		Point bboxPoint = boundingBoxBoundaryPointRelativeToCentre();
		// System.out.println("bbox point: " + bboxPoint);
		int xDisplacement = bboxPoint.x;
		int yDisplacement = bboxPoint.y;

		// relate the tip to world's origin
		int spriteCentreX = (int) (getTheBoundingShape().getBounds2D()
				.getCenterX());
		int spriteCentreY = (int) (getTheBoundingShape().getBounds2D()
				.getCenterY());

		UtilityCircle circ = new UtilityCircle(bboxPoint);
		// now use same angle, but smaller diameter
		double tipDist = Math.sqrt(Math.pow(
				getPointOfShooterTipRelativeToSpriteCentre().getX(), 2)
				+ Math.pow(getPointOfShooterTipRelativeToSpriteCentre().getY(),
						2));
		UtilityCircle circ3 = new UtilityCircle(getAngularDisplacement()
				- circ.getAngle(), tipDist);
		xDisplacement = (int) circ3.getPointOnCircumference().getX();
		yDisplacement = (int) circ3.getPointOnCircumference().getY();

		int tipX = spriteCentreX + xDisplacement;
		int tipY = spriteCentreY + yDisplacement;

		// System.out.println("tip: " + tipX + "\t " + tipY);
		return new Point(tipX, tipY);

	}

	/**
	 * This method returns a point on the perimeter of the bounding box defined
	 * as follows. The origin of the coordinate system is the upper-left corner
	 * of the bounding box. Suppose a line is drawn from the centre of the
	 * bounding box, through the shooterAnchor point and reaching to the edge of
	 * the bounding box. The point at which the line reaches the bounding box is
	 * the point returned by this method
	 * 
	 * @return
	 */
	private Point boundingBoxBoundaryPointRelativeToCentre() {

		// System.out.println("BBox dim: "
		// + getTheBoundingShape().getBounds2D().getWidth() + "\t"
		// + getTheBoundingShape().getBounds2D().getHeight());

		double yCoord = getTheBoundingShape().getBounds2D().getHeight() / 2;
		yCoord = -yCoord;
		// following derivation is based on isolating x in expression:
		// y = mx+b,
		// and using known values:
		// b=0, m=(y2-y1)/(x2-x1).

		// The 2 coords are: (0,0) [the centre point] and
		// (tipRelativeToCentre.getX(),
		// tipRelativeToCentre.getY())
		double xCoord = yCoord
				* getPointOfShooterTipRelativeToSpriteCentre().getX()
				/ getPointOfShooterTipRelativeToSpriteCentre().getY();

		// System.out.println("boundingBoxBoundaryPointRelativeToCentre: "
		// + xCoord + "\t" + yCoord);

		return new Point((int) xCoord, (int) yCoord);
	}

	/**
	 * Returns the coordinates of the shooter tip, relative to the centre of the
	 * sprite
	 * 
	 * @return as described above
	 */
	public Point getPointOfShooterTipRelativeToSpriteCentre() {
		return pointOfShooterTipRelativeToSpriteCentre;
	}

	/**
	 * Sets the coordinates of the shooter tip, relative to the centre of the
	 * sprite
	 * 
	 * @param pointOfShooterTipRelativeToSpriteCentre
	 *            as described above
	 */
	public void setPointOfShooterTipRelativeToSpriteCentre(
			Point pointOfShooterTipRelativeToSpriteCentre) {
		this.pointOfShooterTipRelativeToSpriteCentre = pointOfShooterTipRelativeToSpriteCentre;
	}

	public void rotateLeft() {
		setAngularDisplacement(angularDisplacement - ANGLE_DELTA);
	}

	public void rotateRight() {
		setAngularDisplacement(angularDisplacement + ANGLE_DELTA);
	}

}
