package sprites;

import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.geom.Rectangle2D;

import simulation_Lab12Version.UtilityClass;

public abstract class Sprite {

	private int widthOfDrawingRegion;

	private int heightOfDrawingRegion;

	public int RIGHT_DELTA = 2;
	public int LEFT_DELTA = -5;
	public int DOWN_DELTA = 2;
	public int UP_DELTA = -5;

	private final int MIN_PERTURB = -10;
	private final int MAX_PERTURB = 10;

	// private Point anchorPoint;// = new Point(0, 0);
	private Polygon theBoundingShape;// = new Polygon();

	// indicated whether this sprite has been hit by another (to register state
	// change if this sprite is something that can serve as a target)
	private boolean isHit = false;

	protected Sprite(int widthOfDrawingRegion, int heightOfDrawingRegion) {
		this.widthOfDrawingRegion = widthOfDrawingRegion;
		this.heightOfDrawingRegion = heightOfDrawingRegion;
	}

	public void draw(Graphics2D graphics2D) {
	}

	public void moveLeft() {
	}

	public void moveRight() {
	}

	public void moveUp() {
	}

	public void moveDown() {
	}


	public int getWidthOfDrawingRegion() {
		return widthOfDrawingRegion;
	}

	public void setWidthOfDrawingRegion(int widthOfDrawingRegion) {
		this.widthOfDrawingRegion = widthOfDrawingRegion;
	}

	public int getHeightOfDrawingRegion() {
		return heightOfDrawingRegion;
	}

	public void setHeightOfDrawingRegion(int heightOfDrawingRegion) {
		this.heightOfDrawingRegion = heightOfDrawingRegion;
	}

	protected Point wrapAround(Point currentPoint) {
		int newX = currentPoint.x % getWidthOfDrawingRegion();
		int newY = currentPoint.y % getHeightOfDrawingRegion();
		if (newX < 0)
			newX = newX + getWidthOfDrawingRegion();
		if (newY < 0)
			newY = newY + getHeightOfDrawingRegion();
		return new Point(newX, newY);
	}

	protected Point addJitter(Point initialPoint) {
		// add the jitter now
		int horizontalPerturbValue = UtilityClass.getRandomValue(
				getMinPerturb(), getMaxPeturb());
		int verticalPerturbValue = UtilityClass.getRandomValue(getMinPerturb(),
				getMaxPeturb());
		Point newPoint = new Point(initialPoint.x + horizontalPerturbValue,
				initialPoint.y + verticalPerturbValue);
		// if jitter caused sprite to move off screen, then wrap it around
		newPoint = wrapAround(newPoint);
		return newPoint;
	}

	public int getRightDelta() {
		return RIGHT_DELTA;
	}

	public int getLeftDelta() {
		return LEFT_DELTA;
	}

	public int getDownDelta() {
		return DOWN_DELTA;
	}

	public int getUpDelta() {
		return UP_DELTA;
	}

	public int getMinPerturb() {
		return MIN_PERTURB;
	}

	public int getMaxPeturb() {
		return MAX_PERTURB;
	}

	protected Point getNewPosition(Point currentPoint,
			int horizontalDisplacement, int verticalDisplacement) {
		int newX = (currentPoint.x + horizontalDisplacement);
		int newY = (currentPoint.y + verticalDisplacement);
		Point newPosition = new Point(newX, newY);
		newPosition = wrapAround(newPosition);
		return newPosition;
	}

	public void reposition(int newX, int newY) {
		setAnchorPoint(new Point(newX, newY));
	}

	public Point getAnchorPoint() {
		return new Point((int) getBounds2D().getX(), (int) getBounds2D().getY());
	}

	/**
	 * PRE BoundingBox must be set prior to this
	 * 
	 * @param newAnchorPoint
	 */
	public void setAnchorPoint(Point newAnchorPoint) {
		int transX = newAnchorPoint.x - getCurrentX();
		int transY = newAnchorPoint.y - getCurrentY();
		if (getTheBoundingShape() != null) {
			getTheBoundingShape().translate(transX, transY);
		}
	}

	private int getCurrentX() {
		return (int) getBounds2D().getX();
	}

	private int getCurrentY() {
		return (int) getBounds2D().getY();
	}

	public Polygon getTheBoundingShape() {
		return theBoundingShape;
	}

	public void setTheBoundingShape(Polygon theEnclosingShape) {
		this.theBoundingShape = theEnclosingShape;
	}

	public Rectangle2D getBounds2D() {
		return getTheBoundingShape().getBounds2D();
	}

	public boolean isCollided(Sprite s) {
		boolean cond1 = isHit == false;
		boolean cond2 = getBounds2D().intersects(s.getBounds2D().getX(),
				s.getBounds2D().getY(), s.getBounds2D().getWidth(),
				s.getBounds2D().getHeight());

		// System.out
		// .println("collide " + getBounds2D() + "\t " + s.getBounds2D());
		if (cond2)
			System.out.println("HIT!");

		return cond1 && cond2;
	}

	/**
	 * Causes this sprite to transition to an exploded state. When this sprite
	 * is drawn, the explode behaviour can be specified.
	 * 
	 */
	public void setExploded() {
		isHit = true;
	}

	/**
	 * Causes this sprite to transition to an exploded state. When this sprite
	 * is drawn, the explode behaviour can be specified.
	 * 
	 */
	public boolean isExploded() {
		return isHit;
	}

}
